unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TCOSData = array of packed record
    A, B, C: Byte;
  end;

  TMainForm = class(TForm)
    LogMemo: TMemo;
    Panel2: TPanel;
    CardCombo: TComboBox;
    ControlPanel: TPanel;
    WDGTime: TTimer;
    StatusLabel: TLabel;
    TempLabel: TLabel;
    WDGIRQButton: TButton;
    SoftRestartRadio: TRadioButton;
    MostlySoftRestartRadio: TRadioButton;
    DisableWDGCheck: TCheckBox;
    NoRestartRadio: TRadioButton;
    GoWDGButton: TButton;
    CancelWDGButton: TButton;
    TempButton: TButton;
    StatusButton: TButton;
    AbortButton: TButton;
    DICheck: TCheckBox;
    DIGroup: TGroupBox;
    DILabel0: TLabel;
    DILabel1: TLabel;
    DILabel2: TLabel;
    DILabel3: TLabel;
    DILabel4: TLabel;
    DILabel5: TLabel;
    PollTime: TTimer;
    DOGroup: TGroupBox;
    DOCheck0: TCheckBox;
    DOCheck4: TCheckBox;
    DOCheck3: TCheckBox;
    DOCheck1: TCheckBox;
    DOCheck2: TCheckBox;
    DOWriteButton: TButton;
    AIGroupBox: TGroupBox;
    lblTemp0: TLabel;
    lblRH: TLabel;
    lblADC0: TLabel;
    lblADC1: TLabel;
    lblADC3: TLabel;
    lblADC2: TLabel;
    cbADC: TCheckBox;
    Button2: TButton;
    rgVolume: TRadioGroup;
    GroupBox1: TGroupBox;
    butEnableBattery: TButton;
    butDisableBattery: TButton;
    procedure FormCreate(Sender: TObject);
    procedure AbortButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure WDGIRQButtonClick(Sender: TObject);
    procedure GoWDGButtonClick(Sender: TObject);
    procedure CancelWDGButtonClick(Sender: TObject);
    procedure TempButtonClick(Sender: TObject);
    procedure StatusButtonClick(Sender: TObject);
    procedure CardComboChange(Sender: TObject);
    procedure WDGPetTimer(Sender: TObject);
    procedure WDGWaitTimer(Sender: TObject);
    procedure PollCheckClick(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
    procedure DOWriteButtonClick(Sender: TObject);
    procedure butBuzzerClick(Sender: TObject);
    procedure butEnableBatteryClick(Sender: TObject);
    procedure butDisableBatteryClick(Sender: TObject);
  public
    procedure AddLine(CardNum: Integer; Line: String);
    procedure ReceiveGenIRQ(CardNum: Integer);
    procedure ReceiveCOSData(CardNum: Integer; COSData: TCOSData);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDG;

type
  TCardData = record
    IRQCount: LongWord;
    LastCOSData: TCOSData;
    Log: String;
    WDGPets: LongWord;
  end;

var
  CardData: array of TCardData;

procedure TMainForm.FormCreate(Sender: TObject);
var
  NumCards, CardNum: Integer;
  DeviceID: LongWord;
  NameSize: LongWord;
  Name: String;
  ErrorCode: LongWord;

begin
  CardCombo.Items.Clear;
  NumCards := GetNumCards;
  if NumCards = 0 then begin
    ControlPanel.Visible := False;

    LogMemo.Text := 'No cards found. The cards may not be installed, or they may be installed using a driver other than AIOWDG.sys.';
  end
  else begin
    SetLength(CardData, NumCards);
    for CardNum := 0 to NumCards - 1 do begin
      ErrorCode := QueryCardID(CardNum, @DeviceID);
      if ErrorCode <> ERROR_SUCCESS then begin
        CardCombo.Items.Add('Error ' + IntToStr(ErrorCode) + ' getting data: Card ' + IntToStr(CardNum));
        Continue;
      end;

      case DeviceID of
        $8080, $8081: begin //PCIe-WDG-CSM, PCIe-WDG-CSMA
          WDGInit(CardNum);
        end;
      end;

      NameSize := 256;
      SetString(Name, nil, NameSize);
      ErrorCode := QueryCardName(CardNum, @NameSize, PChar(Name));
      if ErrorCode <> ERROR_SUCCESS then begin
        CardCombo.Items.Add('Error ' + IntToStr(ErrorCode) + ' getting data: Card ' + IntToStr(CardNum));
        Continue;
      end;

      SetLength(Name, NameSize);
      CardCombo.Items.Add(Name);

      CardData[CardNum].Log :=
          'AIOWDG provides special functions for watchdog cards.'#13#10#13#10
        + 'Click "Handle Watchdog IRQ" to handle the next IRQ for the card selected above with the selected action.'#13#10#13#10
        + 'You can also click "Test Watchdog Timing" to set the watchdog timer, pet it a few times, then let it time out, '
        + '"Read Temp" to read the temperature(if installed), '
        + '"Read Status" to read the card''s status byte, '
        + '"Read Digital In" to start/stop reading the card''s various digital inputs, '
        + 'or "Write Digital Out" to write to the card''s various digital outputs.'#13#10#13#10
      ;
    end;
    CardCombo.ItemIndex := 0;
    CardComboChange(CardCombo);
  end;
end;

procedure TMainForm.CardComboChange(Sender: TObject);
var
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;

  LogMemo.Text := CardData[CardNum].Log;
  LogMemo.SelStart := Length(CardData[CardNum].Log);
  LogMemo.SelLength := 1;

  TempLabel.Caption := '';
  StatusLabel.Caption := '';
end;

procedure TMainForm.AbortButtonClick(Sender: TObject);
var
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;
  if AbortRequest(CardNum) = ERROR_SUCCESS then
    AddLine(CardNum, 'Aborting IRQ detection.')
  else
    AddLine(CardNum, 'No IRQ request to abort.')
  ;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  CardNum: Integer;

begin
  {
    Abort all pending IRQ requests. If we don't do this before exiting, later
    when an IRQ comes in or somebody else calls AbortRequest the pending IRQ
    request will attempt to unlock a thread that no longer exists, which is bad.

    (We can AbortRequest for a card with no pending requests, it'll just tell us
    that, and it's easier than remembering which cards have pending requests.)
  }
  for CardNum := 0 to GetNumCards - 1 do
    AbortRequest(CardNum)
  ;
end;

procedure TMainForm.WDGIRQButtonClick(Sender: TObject);
var
  Action: LongWord;
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;

  if SoftRestartRadio.Checked then
    Action := WDG_ACTION_SOFT_RESTART
  else if MostlySoftRestartRadio.Checked then
    Action := WDG_ACTION_MOSTLY_SOFT_RESTART
  else
    Action := WDG_ACTION_IGNORE
  ;

  if DisableWDGCheck.Checked then
    Action := Action + WDG_ACTION_DISABLE
  ;

  WDGHandleIRQ(CardNum, Action);

  AddLine(CardNum, 'Watchdog IRQ handler started, press "Abort IRQ Detect" to cancel.');
end;

procedure TMainForm.GoWDGButtonClick(Sender: TObject);
var
  CardNum: Integer;
  Milliseconds: Double;

begin
  CardNum := CardCombo.ItemIndex;

  Milliseconds := 2 * 1000; //2 sec timeout
  WDGSetTimeout(CardNum, Milliseconds);
  Milliseconds := 1; //infinite reset
  //Milliseconds := 250; //250 ms reset
  WDGSetResetDuration(CardNum, Milliseconds);

  WDGStart(CardNum);

  AddLine(CardNum, 'Watchdog timing started...');
  CardData[CardNum].WDGPets := 0;
  WDGTime.Tag := CardNum;
  WDGTime.OnTimer := WDGPetTimer;
  WDGTime.Interval := 1000;
  WDGTime.Enabled := True;
end;

procedure TMainForm.WDGPetTimer(Sender: TObject);
var
  CardNum: Integer;
  StatusByte: LongWord;

begin
  CardNum := WDGTime.Tag;

  WDGReadStatus(CardNum, StatusByte);
  if (StatusByte and $01) = 0 then begin
    AddLine(CardNum, 'Watchdog timed out early!');
    WDGTime.Enabled := False;
    Exit;
  end;

  Inc(CardData[CardNum].WDGPets);
  if CardData[CardNum].WDGPets > 3 then begin
    AddLine(CardNum, 'Waiting for timeout...');
    WDGTime.OnTimer := WDGWaitTimer;
    WDGTime.Interval := 100;
  end
  else begin
    WDGPet(CardNum);
    AddLine(CardNum, 'Petting watchdog, pet ' + IntToStr(CardData[CardNum].WDGPets) + ' of 3...');
  end;
end;

procedure TMainForm.WDGWaitTimer(Sender: TObject);
var
  CardNum: Integer;
  StatusByte: LongWord;

begin
  CardNum := WDGTime.Tag;

  WDGReadStatus(CardNum, StatusByte);
  if (StatusByte and $01) = 0 then begin
    AddLine(CardNum, 'Watchdog timed out.');
    WDGTime.Enabled := False;
  end;
end;

procedure TMainForm.CancelWDGButtonClick(Sender: TObject);
var
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;
  WDGStop(CardNum);
  WDGTime.Enabled := False;
  AddLine(CardNum, 'Watchdog timing cancelled.');
end;

procedure TMainForm.TempButtonClick(Sender: TObject);
var
  TempF: Double;
  Err: LongWord;

begin
  TempF := -1;
  Err := WDGReadTemp(CardCombo.ItemIndex, TempF);
  if Err = ERROR_SUCCESS then
    TempLabel.Caption := FloatToStr(TempF)
  else
    TempLabel.Caption := 'Error ' + IntToStr(Err) + ' from WDGReadTemp.'
  ;
end;

procedure TMainForm.StatusButtonClick(Sender: TObject);
var
  StatusByte: LongWord;
  Err: LongWord;

begin
  Err := WDGReadStatus(CardCombo.ItemIndex, StatusByte);
  if Err = ERROR_SUCCESS then
    StatusLabel.Caption := IntToHex(StatusByte, 2)
  else
    StatusLabel.Caption := 'Error ' + IntToStr(Err) + ' from WDGReadStatus.'
  ;
end;

procedure TMainForm.AddLine(CardNum: Integer; Line: String);
begin
  CardData[CardNum].Log := CardData[CardNum].Log + Line + #13#10#13#10;
  if CardNum = CardCombo.ItemIndex then CardComboChange(CardCombo);
end;

procedure TMainForm.ReceiveCOSData(CardNum: Integer; COSData: TCOSData);
begin
  CardData[CardNum].LastCOSData := COSData;
  if CardNum = CardCombo.ItemIndex then CardComboChange(CardCombo);
end;

procedure TMainForm.ReceiveGenIRQ(CardNum: Integer);
begin
  Inc(CardData[CardNum].IRQCount);
  if CardNum = CardCombo.ItemIndex then CardComboChange(CardCombo);
end;

procedure TMainForm.PollCheckClick(Sender: TObject);
begin
  PollTime.Enabled := DICheck.Checked or cbADC.Checked;

  DILabel0.Caption := '';
  DILabel1.Caption := '';
  DILabel2.Caption := '';
  DILabel3.Caption := '';
  DILabel4.Caption := '';
  DILabel5.Caption := '';

  lblTemp0.Caption := '';
  lblRH.Caption := '';
  lblADC0.Caption := '';
  lblADC1.Caption := '';
  lblADC2.Caption := '';
  lblADC3.Caption := '';
end;

procedure TMainForm.PollTimeTimer(Sender: TObject);

  function IntToBitDesc(I: Integer): AnsiString;
  begin
    if I = 0 then
      Result := 'low'
    else
      Result := 'HIGH'
    ;
  end;

  function Si7013_TempToCelsius(x: word): double;
  begin
      Result := (175.72*x)/65536.0 - 46.85;
  end;
  function Si7013_HumidityToRH(x: word) : double;
  begin
      Result :=  125.0*x/65536.0 - 6.0;
  //    Result :=  min( max((125.0*x)/65536.0 - 6.0, 0), 100 );
  end;

  function Si7013_CountsToMilliVolts(x: word) : double;
  begin
      Result :=  1.25 * x / 32.767;
  end;

var
  DIData: Byte;
  AIBuf: array of Double;
  L: LongWord;
  buf: array of Byte;
  pADCCounts: PWord;

begin
  // Digital Inputs
  if DICheck.Checked then try
    DI_ReadAll(CardCombo.ItemIndex, @DIData);

    DILabel0.Caption := 'Isolated In 0: ' + IntToBitDesc(DIData and (1 shl 0));
    DILabel1.Caption := 'Isolated In 1: ' + IntToBitDesc(DIData and (1 shl 1));
    DILabel2.Caption := 'Isolated In 2: ' + IntToBitDesc(DIData and (1 shl 2));
    DILabel3.Caption := 'Isolated In 3: ' + IntToBitDesc(DIData and (1 shl 3));
    DILabel4.Caption := 'TTL In 0: ' + IntToBitDesc(DIData and (1 shl 4));
    DILabel5.Caption := 'TTL In 1: ' + IntToBitDesc(DIData and (1 shl 5));
  except
    DICheck.Checked := False;
    raise;
  end;

  // Analog Inputs
  if cbADC.Checked then try
   begin
      SetLength(AIBuf, 6);
      AI_ReadAll(CardCombo.ItemIndex, @AIBuf[0]);
      lblTemp0.Caption := 'Internal Temp: ' + Format('%10.4f', [ AIBuf[0] ]) + 'C';  // internal temp
      lblRH.Caption    := 'Humidity: ' + Format('%10.4f', [ AIBuf[1] ]) + ' %RH';  // relative humidity
      lblADC0.Caption  := '12V Power: ' + Format('%10.4f', [ AIBuf[2] ]) +  ' mV';  // 12V
      lblADC1.Caption  := '3.3V Power: ' + Format('%10.4f', [ AIBuf[3] ]) + ' mV';  // 3.3V
      lblADC2.Caption  := 'External Temp: ' + Format('%10.4f', [ (AIBuf[4] - 460) * 70 / 320 ]) + ' C';  // ext temp
      lblADC3.Caption  := 'Light Sensor: ' + Format('%10.4f', [ AIBuf[5] ]) + ' mV';  // light sensor
    end;
  except
    cbADC.Checked := False;
    raise;
  end;
end;

procedure TMainForm.DOWriteButtonClick(Sender: TObject);
var
  DOData: Byte;

begin
  DOData := 0;
  if DOCheck0.Checked then DOData := DOData or (1 shl 0);
  if DOCheck1.Checked then DOData := DOData or (1 shl 1);
  if DOCheck2.Checked then DOData := DOData or (1 shl 2);
  if DOCheck3.Checked then DOData := DOData or (1 shl 3);
  if DOCheck4.Checked then DOData := DOData or (1 shl 4);

  DO_WriteAll(CardCombo.ItemIndex, DOData);
end;
              
procedure TMainForm.butBuzzerClick(Sender: TObject);
  var
    L: Longword;
    t: Integer;
    volume: Integer;
begin
  L := 0;
  t := (Sender as TButton).tag;
  // 0x8000 to turn off buzzer, 0x8001 to control buzzer
  case rgVolume.ItemIndex of
    0: GenericVendorWrite(CardCombo.ItemIndex, $40, 0    , $8000, L, nil); // turn Siren / Buzzer / Speaker OFF
    1: GenericVendorWrite(CardCombo.ItemIndex, $40, $FFFF, $8001, L, nil); // SET FULL ON SIREN
    2: GenericVendorWrite(CardCombo.ItemIndex, $40, $FF80, $8001, L, nil); // SET SIREN 50% duty cycle
    3: GenericVendorWrite(CardCombo.ItemIndex, $40, $FF20, $8001, L, nil); // Set low duty cycle / quiet
    4: GenericVendorWrite(CardCombo.ItemIndex, $40, $FeFF, $8001, L, nil); // Speaker fastest
    5: GenericVendorWrite(CardCombo.ItemIndex, $40, $07D0, $8001, L, nil); // Speaker ~2KHz
    6: GenericVendorWrite(CardCombo.ItemIndex, $40, $0064, $8001, L, nil); // Speaker 100Hz
  end;//case


end;

procedure TMainForm.butEnableBatteryClick(Sender: TObject);
  var
    L: Longword;
begin
  L := 0;
  GenericVendorWrite(CardCombo.ItemIndex, $40, $8000, $8002, L, nil);
end;

procedure TMainForm.butDisableBatteryClick(Sender: TObject);
  var
    L: Longword;
begin
  L := 0;
  GenericVendorWrite(CardCombo.ItemIndex, $40, $0000, $8002, L, nil);
end;

end.
